@@ -2,28 +2,26 @@
 """
 Converts a mitmproxy dump file to a swagger schema.
 """
-from email import header
-from mitmproxy import io as iom, http
-from mitmproxy.exceptions import FlowReadException
-import pprint
-import sys
-import io
-import json
 import os
+import sys
+import traceback
+from mitmproxy.exceptions import FlowReadException
+import json
 import argparse
 import ruamel.yaml
 import re
-import mitmproxy2swagger.console_util
-from mitmproxy2swagger.har_capture_reader import HarCaptureReader, har_archive_heuristic
-from mitmproxy2swagger.mitmproxy_capture_reader import MitmproxyCaptureReader, mitmproxy_dump_file_huristic
-import mitmproxy2swagger.console_util
+from . import swagger_util
+from .har_capture_reader import HarCaptureReader, har_archive_heuristic
+from .mitmproxy_capture_reader import MitmproxyCaptureReader, mitmproxy_dump_file_huristic
+from . import console_util
+import urllib
 
 def path_to_regex(path):
     # replace the path template with a regex
     path = path.replace('{', '(?P<')
     path = path.replace('}', '>[^/]+)')
     path = path.replace('*', '.*')
-    path = path.replace('/', '\/')
+    path = path.replace('/', '\\/')
     return "^" + path + "$"
 
 
@@ -35,10 +33,24 @@ def strip_query_string(path):
 def set_key_if_not_exists(dict, key, value):
     if key not in dict:
         dict[key] = value
+
+
 def progress_callback(progress):
     console_util.print_progress_bar(progress)
-def main():
 
+
+def detect_input_format(file_path):
+    har_score = har_archive_heuristic(file_path)
+    mitmproxy_score =  mitmproxy_dump_file_huristic(file_path)
+    if 'MITMPROXY2SWAGGER_DEBUG' in os.environ:
+        print('har score: ' + str(har_score))
+        print('mitmproxy score: ' + str(mitmproxy_score))
+    if har_score > mitmproxy_score:
+        return HarCaptureReader(file_path, progress_callback)
+    return MitmproxyCaptureReader(file_path, progress_callback)
+
+
+def main():
     parser = argparse.ArgumentParser(
         description='Converts a mitmproxy dump file or HAR to a swagger schema.')
     parser.add_argument(
@@ -48,15 +60,20 @@ def main():
     parser.add_argument('-p', '--api-prefix', help='The api prefix', required=True)
     parser.add_argument('-e', '--examples', action='store_true',
                         help='Include examples in the schema. This might expose sensitive information.')
+    parser.add_argument('-f', '--format', choices=['flow', 'har'],
+                        help='Override the input file format auto-detection.')
     args = parser.parse_args()
 
     yaml = ruamel.yaml.YAML()
+
     caputre_reader = None
-    # detect the input file type
-    if har_archive_heuristic(args.input) > mitmproxy_dump_file_huristic(args.input):
+    if args.format == 'flow' or args.format == 'mitmproxy':
+        caputre_reader = MitmproxyCaptureReader(args.input, progress_callback)
+    elif args.format == 'har':
         caputre_reader = HarCaptureReader(args.input, progress_callback)
     else:
-        caputre_reader = MitmproxyCaptureReader(args.input, progress_callback)
+        caputre_reader = detect_input_format(args.input)
+
     swagger = None
 
     # try loading the existing swagger file
@@ -70,29 +87,33 @@ def main():
         swagger = ruamel.yaml.comments.CommentedMap({
             "openapi": "3.0.0",
             "info": {
-                "title":  args.input + " Mitmproxy2Swagger",
+                "title": args.input + " Mitmproxy2Swagger",
                 "version": "1.0.0"
             },
         })
     # strip the trailing slash from the api prefix
     args.api_prefix = args.api_prefix.rstrip('/')
 
-    if not 'servers' in swagger or swagger['servers'] is None:
+    if 'servers' not in swagger or swagger['servers'] is None:
         swagger['servers'] = []
+
     # add the server if it doesn't exist
     if not any(server['url'] == args.api_prefix for server in swagger['servers']):
         swagger['servers'].append({
             "url": args.api_prefix,
             "description": "The default server"
         })
-    if not 'paths' in swagger or swagger['paths'] is None:
+
+    if 'paths' not in swagger or swagger['paths'] is None:
         swagger['paths'] = {}
+
     if 'x-path-templates' not in swagger or swagger['x-path-templates'] is None:
         swagger['x-path-templates'] = []
 
     path_templates = []
     for path in swagger['paths']:
         path_templates.append(path)
+
     # also add paths from the the x-path-templates array
     if 'x-path-templates' in swagger and swagger['x-path-templates'] is not None:
         for path in swagger['x-path-templates']:
@@ -102,10 +123,10 @@ def main():
     new_path_templates = []
 
     path_template_regexes = [re.compile(path_to_regex(path))
-                            for path in path_templates]
+                             for path in path_templates]
+
     try:
         for f in caputre_reader.captured_requests():
-            
             # strip the api prefix from the url
             url = f.get_url()
             if not url.startswith(args.api_prefix):
@@ -130,42 +151,66 @@ def main():
             set_key_if_not_exists(
                 swagger['paths'], path_template_to_set, {})
 
-        
-            set_key_if_not_exists(swagger['paths'][path_template_to_set], method,  {
+            set_key_if_not_exists(swagger['paths'][path_template_to_set], method, {
                 'summary': swagger_util.path_template_to_endpoint_name(method, path_template_to_set),
-            
                 'responses': {}
             })
+
             params = swagger_util.url_to_params(url, path_template_to_set)
+
             if params is not None and len(params) > 0:
                 set_key_if_not_exists(swagger['paths'][path_template_to_set][method], 'parameters', params)
+
             if method not in ['get', 'head']:
                 body = f.get_request_body()
                 if body is not None:
                     body_val = None
+                    content_type = None
                     # try to parse the body as json
                     try:
                         body_val = json.loads(f.get_request_body())
+                        content_type = 'application/json'
+                    except UnicodeDecodeError:
+                        pass
                     except json.decoder.JSONDecodeError:
                         pass
+                    if content_type is None:
+                        # try to parse the body as form data
+                        try:
+                            body_val_bytes = dict(urllib.parse.parse_qsl(body, encoding='utf-8', keep_blank_values=True))
+                            body_val = {}
+                            did_find_anything = False
+                            for key, value in body_val_bytes.items():
+                                did_find_anything = True
+                                body_val[key.decode('utf-8')] = value.decode('utf-8')
+                            if did_find_anything:
+                                content_type = 'application/x-www-form-urlencoded'
+                            else:
+                                body_val = None
+                        except UnicodeDecodeError:
+                            pass
+                        
                     if body_val is not None:
                         content_to_set = {
                             'content': {
-                                'application/json': {
+                                content_type: {
                                     'schema': swagger_util.value_to_schema(body_val)
                                 }
                             }
                         }
                         if args.examples:
-                            content_to_set['content']['application/json']['example'] = swagger_util.limit_example_size(
+                            content_to_set['content'][content_type]['example'] = swagger_util.limit_example_size(
                                 body_val)
                         set_key_if_not_exists(
                             swagger['paths'][path_template_to_set][method], 'requestBody', content_to_set)
+
             # try parsing the response as json
             response_body = f.get_response_body()
             if response_body is not None:
                 try:
                     response_json = json.loads(response_body)
+                except UnicodeDecodeError:
+                    response_json = None
                 except json.decoder.JSONDecodeError:
                     response_json = None
                 if response_json is not None:
@@ -185,7 +230,19 @@ def main():
 
     except FlowReadException as e:
         print(f"Flow file corrupted: {e}")
-
+        traceback.print_exception(*sys.exc_info())
+        print(f"{console_util.ANSI_RED}Failed to parse the input file as '{caputre_reader.name()}'. ")
+        if not args.format:
+            print(f"It might happen that the input format as incorrectly detected. Please try using '--format flow' or '--format har' to specify the input format.{console_util.ANSI_RESET}")
+        sys.exit(1)
+    except ValueError as e:
+        print(f"ValueError: {e}")
+        # print stack trace
+        traceback.print_exception(*sys.exc_info())
+        print(f"{console_util.ANSI_RED}Failed to parse the input file as '{caputre_reader.name()}'. ")
+        if not args.format:
+            print(f"It might happen that the input format as incorrectly detected. Please try using '--format flow' or '--format har' to specify the input format.{console_util.ANSI_RESET}")
+        sys.exit(1)
 
     new_path_templates.sort()
 
@@ -240,5 +297,7 @@ def main():
     with open(args.output, 'w') as f:
         yaml.dump(swagger, f)
     print("Done!")
+
+
 if __name__ == "__main__":
     main()
